using TP;

namespace MyWC
{
  /// <summary>
  /// Workflow Controller
  /// </summary>
public class MyWC : TP.WC
{
  /// <summary>
  /// Resource Manager for Each type of resource
  /// </summary>
  static TP.RM Flights;
  static TP.RM Rooms;
  static TP.RM Cars;
  static TP.TM TransactionManager;


  public void Commit(Transaction context)
    {
      TransactionManager.Commit(context );		
    }
  public void Abort (Transaction context)
    {
      TransactionManager.Abort(context);		
    }
		
		

  /// <summary>
  /// 
  /// </summary>
  /// <param name="c">Customer</param>
  /// <param name="flights">array of flight names</param>
  /// <param name="location">room location if room is true</param>
  /// <param name="car">true if request is for a car</param>
  /// <param name="room">true if request is for a room</param>
  /// <returns>price of reservation</returns>
  public bool ReserveItinerary( TP.Customer c, int [] flights, string location, bool car, bool room )
    {
      
      TP.Transaction tid = TransactionManager.Start();
      try
	{
	  if( car )
	    Cars.Reserve( tid, c, new TP.Item( location ));
	  if( room )
	    Rooms.Reserve( tid, c, new TP.Item( location ));
	  foreach( int flight in flights )
	    Flights.Reserve( tid, c, new TP.Item( flight.ToString() ));
	  Cars.Commit( tid );
	  Flights.Commit( tid );
	  Rooms.Commit( tid );
	}
      catch( System.Exception e )
	{
	  Cars.Abort( tid );
	  Flights.Abort( tid );
	  Rooms.Abort( tid );
	  throw e;
	}
      return true;
    }
  
  /***************Client interface methods**************************/
  public bool AddFlight(Transaction context, Item i, int count, int price ){
    return Flights.Add( context, i, count, price );
  }
  
 public  bool DeleteFlight(Transaction context,Item i){
    return Flights.Delete(context,i);
  }
  public bool AddRooms(Transaction context, Item i, int count, int price ){
    return Rooms.Add( context, i, count, price );
  }
  public bool DeleteRooms(Transaction context, Item i){
    return Rooms.Delete(context,i);
  }
  public bool AddCars(Transaction context, Item i, int count, int price ){
    return Cars.Add( context, i, count, price );
  }
 public  bool DeleteCars(Transaction context, Item i){
    return Rooms.Delete(context,i);
  }
 public int  QueryFlight(Transaction context, Item i){
    return Flights.Query(context,i);
  }
  public int  QueryFlightPrice(Transaction context, Item i){
    return Flights.QueryPrice(context,i);
  }

  public int  QueryRooms(Transaction context, Item i){
    return Rooms.Query(context,i);
  }
  public int  QueryRoomsPrice(Transaction context, Item i){
     return Rooms.QueryPrice(context,i);
  }
  public int  QueryCars(Transaction context, Item i){
    return Cars.Query(context, i);
  }
  public int  QueryCarsPrice(Transaction context, Item i){
    return Cars.QueryPrice(context, i);
  }
  public string  QueryCustomerInfo(TP.Transaction context,TP.Customer c){
    string s = Flights.QueryCustomerInfo(context, c);
    s += Cars.QueryCustomerInfo(context, c);
    s += Rooms.QueryCustomerInfo(context, c);
    return s;
  }
  public int  QueryCustomer(TP.Transaction context,TP.Customer c){
    int s = Flights.QueryCustomer(context, c);
    s += Cars.QueryCustomer(context, c);
    s += Rooms.QueryCustomer(context, c);
    return s;
  }




  /**********************************************************/

  static void Main( string [] args )
    {
      int price;
      System.Runtime.Remoting.Channels.ChannelServices.RegisterChannel( new System.Runtime.Remoting.Channels.Http.HttpChannel());
      TransactionManager  = (TP.TM) System.Activator.GetObject( typeof( TP.TM ), "http://localhost:8081/TM.soap");
      Flights = (TP.RM) System.Activator.GetObject( typeof( TP.RM ), "http://localhost:8082/RM.soap");
      Rooms = (TP.RM) System.Activator.GetObject( typeof( TP.RM ), "http://localhost:8083/RM.soap");
      Cars = (TP.RM) System.Activator.GetObject( typeof( TP.RM ), "http://localhost:8084/RM.soap");
      TP.WC WC= new MyWC();
      /* try
	{
	  Customer cust=new TP.Customer();
	  //book flights 198,199
	  WC.ReserveItinerary(cust, new  int[]{ 198, 199 }, "Boston", true, true );
	  price=WC.QueryCustomer( null ,cust);
	  System.Console.WriteLine( string.Format("Itinerary booked for {0} dollar", price ));
	}
      catch( System.Exception e )
	{
	  System.Console.WriteLine( string.Format( "Reservation failed: {0}", e ));
	  }*/
      Flights.Add( null, new TP.Item(200), 100, 1000 );
      Flights.Add( null, new TP.Item( 201), 100, 1000 );
      Rooms.Add( null, new TP.Item( "Boston" ), 100, 200 );
      Cars.Add( null, new TP.Item( "Boston" ), 100, 200 );
      Customer cust1=new TP.Customer();
     WC.ReserveItinerary(cust1, new  int[]{ 200, 201 }, "Boston", true, true );
     price = WC.QueryCustomer(null,cust1);
      System.Console.WriteLine( string.Format( "Itinerary booked for {0} dollar", price ));

      while( true )
	System.Threading.Thread.Sleep( 100000 );
    }
}
}
