namespace TP
{
/// <summary>
/// This namespace provides the interfaces that define this project
/// We also provide files implementing some limited functionality 
/// for these interfaces. You are welcome to write those yourself if you
/// prefer so.
/// </summary>
  [System.Serializable()]
	public class Transaction
	{
		/// <summary>
		/// Id for the transaction
		/// </summary>
		public System.Guid Id;
		/// <summary>
		/// Constructor
		/// </summary>
		public Transaction()
		{
			this.Id = System.Guid.NewGuid();
		}

	}

	/// <summary>
	/// Class for the customer
	/// </summary>
  [System.Serializable()]
	public class Customer
	{
		public System.Guid Id;
		/// <summary>
		/// Constructor. Generates a new Id for the customer
		/// </summary>
		public Customer()
		{
			this.Id = new System.Guid();
		}
	}

	/// <summary>
	/// Class for a reservable Item
	/// </summary>
  [System.Serializable()]
	public class Item
	{
		/// <summary>
		/// </summary>
		public string Name;
		public Item( string name )
		{
			this.Name = name;
		}
		//flight number
		public Item(int name )
		{
			this.Name = name.ToString();
		}
	}

	/// <summary>
	/// interface for resources
	/// </summary>
	public interface RS{}

	/// <summary>
	/// Lock manager interface
	/// </summary>
	public interface LM
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="context"></param>
		/// <param name="resource"></param>
		void LockForRead( Transaction context, RS resource);
		/// <summary>
		/// 
		/// </summary>
		/// <param name="context"></param>
		/// <param name="resource"></param>
		void LockForWrite( Transaction context, RS resource);
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="context"></param>
		void UnlockAll( Transaction context );
	}

	/// <summary>
	/// Transaction manager interface
	/// </summary>
	public interface TM
	{
		Transaction  Start();
		/// <summary>
		/// 
		/// </summary>
		/// <param name="context"></param>
		void Commit( Transaction context );
		/// <summary>
		/// 
		/// </summary>
		/// <param name="context"></param>
		void Abort( Transaction context );
		/// <summary>
		/// 
		/// </summary>
		/// <param name="context"></param>
		void Enlist( Transaction context );
	}

	/// <summary>
	/// Workflow controller interface
	/// </summary>
	public interface WC
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="c"></param>
		/// <param name="flights"></param>
		/// <param name="location"></param>
		/// <param name="car"></param>
		/// <param name="room"></param>
		/// <returns>success</returns>
		bool ReserveItinerary( TP.Customer c, int[] flights, string location, bool car, bool room );
		
		//communication with the RM will happen through the WC...eventually
		void Commit(Transaction context);
		void Abort(Transaction context);
		
		//this would result in a call to Add to the RM that handles flights
		//the name of the item is usually the name of the location
		bool AddFlight(Transaction context, Item i, int count, int price );
		bool DeleteFlight(Transaction context, Item i);
		bool AddRooms(Transaction context, Item i, int count, int price );
		bool DeleteRooms(Transaction context, Item i);
		bool AddCars(Transaction context, Item i, int count, int price );
		bool DeleteCars(Transaction context, Item i);
		int  QueryFlight(Transaction context, Item i);
		int  QueryFlightPrice(Transaction context, Item i);
		int  QueryRooms(Transaction context, Item i);
		int  QueryRoomsPrice(Transaction context, Item i);
		int  QueryCars(Transaction context, Item i);
		int  QueryCarsPrice(Transaction context, Item i);
		string  QueryCustomerInfo(TP.Transaction context,TP.Customer c);
		int  QueryCustomer(TP.Transaction context,TP.Customer c);
		
	}

	/// <summary>
	/// Resource Manager Interface
	/// </summary>
	public interface RM
	{
	/// <summary>
	/// Transaction Boundaries
	/// </summary>
	/// <param name="context"></param>
 
		void Enlist( Transaction context );
		/// <summary>
		/// 
		/// </summary>
		/// <param name="context"></param>
		void Commit( Transaction context );
		/// <summary>
		/// 
		/// </summary>
		/// <param name="context"></param>
		void Abort( Transaction context );
	
		/// <summary>
		/// Populate Database
		/// This method is equivalent to the addXXX in the Java Interface
		/// </summary>
		/// <param name="context"></param>
		/// <param name="i"></param>
		/// <param name="count"></param>
		/// <param name="price"></param>
		/// <returns>success</returns>
		bool Add( Transaction context, Item i, int count, int price );
		bool Delete( Transaction context, Item i );
	
	/// <summary>
	/// Query: equivalent to QueryCars, QueryFlights, QueryRooms in the Java interface
	/// </summary>
	/// <param name="context"></param>
	/// <param name="i"></param>
	/// <returns></returns>
 
		int Query( Transaction context, Item i );
		/// <summary>
		/// 
		/// </summary>
		/// <param name="context"></param>
		/// <param name="i"></param>
		/// <returns></returns>
		int QueryPrice( Transaction context, Item i );
		string QueryCustomerInfo( Transaction context, Customer c );
		/// <summary>
		/// 
		/// </summary>
		/// <param name="context"></param>
		/// <param name="c"></param>
		/// <returns></returns>
		int QueryCustomer( Transaction context, Customer c );
		/// <summary>
		///  Reservation
		/// </summary>
		/// <param name="context"></param>
		/// <param name="c"></param>
		/// <param name="i"></param>
		/// <returns>sucess</returns>

		bool Reserve( Transaction context, Customer c, Item i ); 
		/// <summary>
		/// 
		/// </summary>
		/// <param name="context"></param>
		/// <param name="c"></param>
		void DropReservations( Transaction context, Customer c );
		
	//
	// technical/testing interface
	//
	/// <summary>
		///  Shutdown gracefully.
		/// When this RM restarts, it should not attempt to recover its state if
		/// the client called shutdown to terminate it.
	/// </summary>
	
		void Shutdown();
/// <summary>
		/// Call exit after a specified number of disk writes.
		///Support for this method requires a wrapper around the system's 
		/// write to disk command that decrements the counter set 
		/// by this method.
		/// This counter should default to 0, which implies that the wrapper
		///will do nothing.  If the count is non-zero, the wrapper should decrement								     
		///the counter, see if it becomes zero, and if so, call exit(), otherwise
		/// This method is not part of a transaction.
/// </summary>
/// <param name="diskWritesToWait"></param>
	
		void SelfDestruct(int diskWritesToWait);
	}
}
